
/*
 *   OpenJupes v1.1 
 *
 *   Server and Nickname Jupe Service for Ratbox v2+services.
 *   Copyright (C) 2005 Thomas Carlsson, Joost Vunderink and Alan Milford
 *
 *   NOTE: This service depends heavily on the +services patch by Lee
 *   Hardy. It will not compile without it.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */



#include "stdinc.h"
#include "client.h"
#include "irc_string.h"
#include "send.h"
#include "numeric.h"
#include "ircd.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "s_serv.h"
#include "hash.h"
#include "s_conf.h"
#include "hook.h"


#include "event.h"

#include "services.h"


#include <time.h>
#include <string.h>

#include "oj_main.h"

static int s_fakeserver_squit(struct Client *);

struct SVCMessage rehash_msgtab = {
    "REHASH", 0,
    "REHASH\n"
        "Reloads the OpenJupes config file.",
    s_handle_rehash
};

struct SVCMessage jupe_msgtab = {
    "JUPE", 2,
    "JUPE <server> [reason]\n"
        "Votes to add server jupe.",
    s_handle_jupe
};

struct SVCMessage dejupe_msgtab = {
    "DEJUPE", 2,
    "DEJUPE <server> [reason]\n"
        "Votes to remove server jupe.",
    s_handle_dejupe
};

struct SVCMessage unjupe_msgtab = {
    "UNJUPE", 2,
    "UNJUPE <server> [reason]\n"
        "Votes to remove server jupe.",
    s_handle_dejupe
};

struct SVCMessage help_msgtab = {
    "HELP", 0,
    "HELP\n"
       "Lists available commands.",
    s_handle_help
};

struct SVCMessage history_msgtab = {
    "HISTORY", 1,
    "HISTORY <length>\n"
       "Returns last <n> history items.",
    s_handle_history
};

struct SVCMessage status_msgtab = {
    "STATUS", 0,
    "STATUS\n"
       "Shows current status.",
    s_handle_status
};

/* We call this once per minute .. */
void OJ_Timer( void* v )
{
    OJ_ExpireVotes();
}

/* 
 * Broadcasts "text" to all operators on the visible network 
 */
void OJ_IRCD_BroadcastAllOperators( char* text )
{
    struct Client *target_p;
    dlink_node *ptr;

    DLINK_FOREACH(ptr, global_client_list.head)
    {
        target_p = ptr->data;

        if(!IsPerson(target_p) || !IsOper(target_p))
            continue;

    /*  sendto_anywhere(target_p, mainclient_p->client_p, "PRIVMSG", ":%s", text); */

        svc_message( mainclient_p, target_p, SVC_PRIVMSG, "%s", text );

    }
}

/*
 * Sends "text" to a target client using ircd's native target structure.
 */
void OJ_IRCD_BroadcastNative( struct IRCD_Data* oper_p, char* text )
{
    struct Client* target_p = (struct Client*)oper_p->data;

        if ( !IsPerson( target_p ) || !IsOper( target_p ) || !text )
                return;

    svc_message( mainclient_p, target_p, SVC_PRIVMSG, "%s", text );

}

/*
 * Sends "text" to a target client using a nickname (INEFFICIENT, SHOULD BE REMOVED IN V1.1)
 */
void OJ_IRCD_BroadcastSingleOperator( char* oper, char* text )
{
    struct Client *target_p;

    if ( (target_p = find_person(oper) ) != NULL)
    {
        if(!IsOper(target_p))
            return;

        svc_message( mainclient_p, target_p, 1, "%s", text );
    }
    return;
}

/*
 * Sends "text" to the local operators.
 */
void OJ_IRCD_BroadcastLocal( char* text )
{
    sendto_realops_flags( UMODE_ALL, L_ALL, "Broadcast local: %s", text );
}

void OJ_IRCD_CreateFakeClient( struct OJ_nickjupe* nickjupe )
{
    if ( nickjupe->data )
    {
        /* Already created? */
/*      OJ_IRCD_BroadcastLocal( "Debug: ATTEMPTED TO CREATE NICK JUPE WITH DATA != NULL" );*/
    }
    else
    {
        nickjupe->data = (void*)create_fake_client( nickjupe->nick, nickjupe->user, nickjupe->host, nickjupe->gecos, 0 );
    }
}

void OJ_IRCD_DeleteFakeClient( struct OJ_nickjupe* nickjupe )
{
    if ( nickjupe->data )
    {
            destroy_fake_client( (struct Client *)nickjupe->data );
        nickjupe->data = NULL;
    }
}

void OJ_IRCD_CreateFakeServer( struct OJ_serverjupe* serverjupe )
{
    if ( serverjupe->data )
    {
        OJ_IRCD_BroadcastLocal( "Debug: ATTEMPTED TO CREATE SERVER JUPE WITH DATA != NULL" );
    }
    else
    {
        /* If this ircd is connected, let's squit it first.. */
        struct Client* target_p = find_server( NULL, serverjupe->server );
        if ( target_p )
        {
            exit_client( mainclient_p->client_p, target_p, mainclient_p->client_p, "Server juped" );
        }

        serverjupe->data = (void*)create_fake_server( serverjupe->server, "Server juped", 1 );
    }
}

void OJ_IRCD_DeleteFakeServer( struct OJ_serverjupe* serverjupe )
{
    if ( serverjupe->data )
    {
        destroy_fake_server( (struct Client*)serverjupe->data, 1 );
        serverjupe->data = NULL;
    }
}

static struct OJ_Conf conf;
static int fullinit = 0;

static int modinit(void)
{
    fullinit = 0;

    OJ_Initialize();
    CONF_Load( &conf );

    if ( !conf.ok )
    {
        /* Should we halt here if conf load failed? */
        OJ_IRCD_BroadcastLocal( "Modinit: Fatal errors encountered in reading the conf file. Please fix the conf, unload and reload the module." );
        OJ_Deinitialize();
        return 1; /* I guess */
    }

    hook_add_hook("fake_squit", (hookfn) s_fakeserver_squit);

    mainclient_p = create_service( conf.client.nick, conf.client.user, conf.client.host, conf.client.gecos, 1 );

    /* bind commands to service */
    svc_add_cmd(mainclient_p, &rehash_msgtab);
    svc_add_cmd(mainclient_p, &help_msgtab);
    svc_add_cmd(mainclient_p, &jupe_msgtab);
    svc_add_cmd(mainclient_p, &dejupe_msgtab);
    svc_add_cmd(mainclient_p, &unjupe_msgtab);
    svc_add_cmd(mainclient_p, &status_msgtab);
    svc_add_cmd(mainclient_p, &history_msgtab);
      
    OJ_CreateNickJupes( &conf );

    OJ_LoadDB();

    eventAdd( "OJ_Timer", OJ_Timer, NULL, 60 );

    fullinit = 1;
    
    return 0;
}

static void moddeinit(void)
{
    if ( !fullinit ) return;

    eventDelete( OJ_Timer, NULL );

    OJ_DeleteNickJupes( &conf );

    destroy_service( mainclient_p );

    OJ_SaveDB();
    OJ_Deinitialize();
    CONF_Free( &conf );

    hook_del_hook("fake_squit", (hookfn) s_fakeserver_squit);

    fullinit = 0;
}

const char *_version = "$Revision: 1.3 $";

int
s_handle_jupe(struct Client *source_p, struct Client *target_p, int parc, const char *parv[])       
{
    struct OJ_voter ojv;
    char reason[512];
    int i;
        
    strcpy( ojv.nick, source_p->name );
    strcpy( ojv.user, source_p->username );
    strcpy( ojv.host, source_p->host );
    strcpy( ojv.server, source_p->user->server );
    ojv.timestamp = time( NULL );

    /* Is this aspiring voter on an a server to ignore? */
    if ( CONF_IsIgnore( &conf, ojv.server ) )
    {
        svc_message( mainclient_p, source_p, SVC_PRIVMSG, "JUPE: Request rejected: Your server is on my ignore list." );
        return 0;
    }

    if ( ( parc < 2 ) || ( strlen( parv[1] ) > HOSTLEN ) )
    {
        svc_message( mainclient_p, source_p, SVC_PRIVMSG, "Syntax: JUPE <server> [reason]" );
        return 0;
    }

    if ( !irccmp( parv[1] , me.name ) )
    {
        /* They want me to jupe myself? */
        svc_message( mainclient_p, source_p, SVC_PRIVMSG, "JUPE: Request rejected: Be serious." );
        return 0;
    }

    /* Set the reason to the second argument, if it has been given; otherwise,
     * make it empty. */
    reason[0] = 0;
    if (parc > 2) {
	for (i = 2; i < parc; i++) {
	    strcat(reason, parv[i]);
	    strcat(reason, " ");
	}
    }
    
    if ( IsAdmin( source_p ) ) 
    {
        ojv.score = 5;
    }
    else
    {
        if ( CONF_IsAdmin( &conf, ojv.user, ojv.host, ojv.server ) )
        {   
            ojv.score = 5;
        }
        else
        {
            ojv.score = 3;
        }
    }

    OJ_Vote( (char *)parv[1], reason, &ojv );

    return 0;
}

int
s_handle_dejupe(struct Client *source_p, struct Client *target_p, int parc, const char *parv[])
{

    struct OJ_voter ojv;

    strcpy( ojv.nick, source_p->name );
    strcpy( ojv.user, source_p->username );
    strcpy( ojv.host, source_p->host );
    strcpy( ojv.server, source_p->user->server );
    ojv.timestamp = time( NULL );

    /* Is this aspiring voter on an a server to ignore? */
    if ( CONF_IsIgnore( &conf, ojv.server ) )
    {
        svc_message( mainclient_p, source_p, SVC_PRIVMSG, "UNJUPE: Request rejected: Your server is on my ignore list." );
        return 0;
    }

    if (parc < 2)
    {
        svc_message( mainclient_p, source_p, SVC_PRIVMSG,  "Syntax: DEJUPE <server> :[reason]" );
        return 0;
    }

    if ( IsAdmin( source_p ) ) 
    {
        ojv.score = -5;
    }
    else
    {
        if ( CONF_IsAdmin( &conf, ojv.user, ojv.host, ojv.server ) )
        {   
            ojv.score = -5;
        }
        else
        {
            ojv.score = -3;
        }
    }

    OJ_Vote( (char *)parv[1], (char *)parv[2], &ojv );

    return 0;
}

void send_command_list(struct Client *target_p)
{
    struct SVCMessage *mptr;
    char cmds[IRCLINELEN];
    cmds[0] = 0;
    dlink_node *ptr;
    
    if (target_p == NULL)
        return;
        

    DLINK_FOREACH(ptr, mainclient_p->commands.head)
    {
        mptr = (struct SVCMessage *)ptr->data;
        
        strcat(cmds, mptr->cmd);
        strcat(cmds, " ");  
    }
    
    svc_message(mainclient_p, target_p, SVC_PRIVMSG, "List of commands:");
    svc_message(mainclient_p, target_p, SVC_PRIVMSG, "%s", cmds);
}

int s_handle_help(struct Client *source_p, struct Client *target_p, int parc, const char *parv[])
{
    struct SVCMessage *mptr;
    char *parv2[OJ_MAXPARA];
    int i, parc2;
    char *help;
    
    if (parc < 2)
    {
        send_command_list(source_p);
    }
    else
    {
        if ((mptr = svc_get_cmd(mainclient_p, (char *)parv[1])) != NULL)
        {
            help = LOCAL_COPY((char *)mptr->help);
            parc2 = tokenize_string_separator(help, parv2, '\n');
            for (i = 0; i < parc2; i++) 
            {
           
                svc_message(mainclient_p, source_p, SVC_PRIVMSG, "%s", parv2[i]);

            } 
            
        }
        else
            send_command_list(source_p);
    }
    
    return 0;
    
}

int 
s_handle_history(struct Client *source_p, struct Client *target_p, int parc, const char *parv[])
{
            struct IRCD_Data native;
            native.data = (void*)source_p;

            OJ_GetHistory( &native, 5 );
      
            return 0;
}

int
s_handle_status(struct Client *source_p, struct Client *target_p, int parc, const char *parv[])
{
        struct IRCD_Data native;
        native.data = (void*)source_p;

        OJ_GetActive( &native );

        return 0;
}

int 
s_handle_rehash(struct Client *source_p, struct Client *target_p, int parc, const char *parv[])
{
        struct Client *admin_p;
        dlink_node *ptr;

        DLINK_FOREACH(ptr, oper_list.head)         
        {
            admin_p = (struct Client *)ptr->data;
            if ( admin_p == source_p && IsAdmin( admin_p ) )
            {
                CONF_Rehash( &conf );
                svc_message( mainclient_p, source_p, SVC_PRIVMSG, "Rehashing config file." );
                return 0;
            }
        }
        return 0;
}

static int
s_fakeserver_squit(struct Client *target_p)
{
    if ( OJ_IsJuped( target_p->name ) )
    {
            sendto_realops_flags(UMODE_ALL, L_ALL, "A server jupe was squit!");
    }

    return 0;
}

int tokenize_string(char *string, char *parv[OJ_MAXPARA])
{
    return tokenize_string_separator(string, parv, ' ');
}

int tokenize_string_separator(char *string, char *parv[OJ_MAXPARA], char separator)
{
    char *p;
    char *buf = string;
    int x = 0;

    parv[x] = NULL;

    while (*buf == separator) /* skip leading spaces */
        buf++;

    if (*buf == '\0') /* ignore all-separator args */
        return x;

    do
    {
        parv[x++] = buf;
        parv[x]   = NULL;

        if ((p = strchr(buf, separator)) != NULL)
        {
            *p++ = '\0';
            buf  = p;
        }
        else
        {
            return x;
        }

        /* Treat multiple separators as one */
        while (*buf == separator)
            buf++;

        if (*buf == '\0')
            return x;
    } while (x < OJ_MAXPARA - 1);

    parv[x++] = p;
    parv[x]   = NULL;
    return x;
}


DECLARE_MODULE_AV1( oj_module, modinit, moddeinit, NULL, NULL, NULL, "$Revision: 1.3 $" );
